// GetFileClient.java

import java.net.*;
import java.io.*;

public class GetFileClient {

	public static void main(String[] args) throws IOException {
		/*
		 * Come argomenti si devono passare un nome di host valido e una porta
		 *  
		 */
		InetAddress addr = null;
		int port = -1;

		/* controllo argomenti */
		try {
			if (args.length == 2) {
				addr = InetAddress.getByName(args[0]);
				port = Integer.parseInt(args[1]);
			} else {
				System.out.println("Usage: java GetFileClient serverAddr serverPort");
				System.exit(1);
			}
		} //try
		// Per esercizio si possono dividere le diverse eccezioni
		catch (Exception e) {
			System.out.println("Problemi, i seguenti: ");
			e.printStackTrace();
			System.out.println("Usage: java GetFileClient serverAddr serverPort");
			System.exit(2);
		}

		// oggetti utilizzati dal client per la comunicazione e la lettura del file
		// locale
		Socket socket = null;
		DataInputStream inSock = null;
		DataOutputStream outSock = null;

		/* comunicazione col cliente, da console */
		// creazione stream di input da tastiera
		BufferedReader stdIn = new BufferedReader(new InputStreamReader(System.in));
		String nomeDirettorio = null;
		System.out
				.print("\n^D(Unix)/^Z(Win)+invio per uscire, altrimenti immetti il nome del direttorio: ");

		try {
			if ((nomeDirettorio = stdIn.readLine()) != null) {

				// creazione socket e stream di input/output su socket
				try {
					socket = new Socket(addr, port);
					// setto il timeout per non bloccare indefinitivamente il client
					socket.setSoTimeout(30000);
					System.out.println("Creata la socket: " + socket);
					inSock = new DataInputStream(socket.getInputStream());
					outSock = new DataOutputStream(socket.getOutputStream());
				} catch (IOException ioe) {
					System.out
							.println("Problemi nella creazione degli stream su socket: ");
					ioe.printStackTrace();
					System.out
							.print("\n^D(Unix)/^Z(Win)+invio per uscire, solo invio per continuare: ");
					// il client esce in modo anomalo
					System.exit(1);
				} catch (Exception e) {
					System.out.println("Problemi nella creazione della socket: ");
					e.printStackTrace();
					System.out
							.print("\n^D(Unix)/^Z(Win)+invio per uscire, solo invio per continuare: ");
					// il client esce in modo anomalo
					System.exit(2);
				}

				// trasmissione del nome e realizzazione protocollo col server
				try {
					// invio il nome del file richiesto
					outSock.writeUTF(nomeDirettorio);
					System.out
							.println("Inviato il nome del direttorio " + nomeDirettorio);
					String nomeFileRicevuto;
					long numeroByte;
					File fileCorr;
					FileOutputStream outFileCorr;
					// leggo il nome del file
					while ( (nomeFileRicevuto = inSock.readUTF())!=null) {
						if (!nomeFileRicevuto.equals("?")) {
							// leggo il numero di byte
							numeroByte = inSock.readLong();
							// controllo se il file esiste, se non esiste lo creo,
							// altrimenti
							// torno errore
							fileCorr = new File(nomeFileRicevuto);
							if (fileCorr.exists())
								fileCorr.delete();
							outFileCorr = new FileOutputStream(nomeFileRicevuto);
							System.out.println("Scrivo il file " + nomeFileRicevuto + " di "
									+ numeroByte + " byte");
							// trasferimento file
							FileUtility.trasferisci_N_byte_file_binario(inSock,
									new DataOutputStream(outFileCorr), numeroByte);
							// chiusura file
							outFileCorr.close();
						} else {
							System.err.println("Questo direttorio: " + nomeFileRicevuto
									+ " non e' presente sul server esco");
						  socket.close();
						  System.exit(3);
						}
					} //while
				}
				/*
				 * NOTA: in caso di raggiungimento dell'EOF, la readUTF
				 * lancia una eccezione che viene gestita qui sotto
				 * chiudendo la socket e terminando il client con successo.
				 */
				catch (EOFException e) {
					System.out.println("Raggiunta la fine delle ricezioni, chiudo...");
					//e.printStackTrace();
					// finito il ciclo di ricezioni termino la comunicazione 
					socket.close();
					// Esco con indicazione di successo
					System.out.println("GetFileClient: termino...");
					System.exit(0);
				} 
				// qui potete dirimere i diversi casi di errore
				catch (Exception e) {
					System.out.println("Problemi nell'invio del nome di "
							+ nomeDirettorio + ": ");
					e.printStackTrace();
					socket.close();
					// il client esce in modo anomalo
					System.exit(3);
				}
			}// if			
		}
		// qui catturo le eccezioni non catturate all'interno del while
		// quali per esempio la caduta della connessione con il server
		// in seguito alle quali il client termina l'esecuzione
		catch (Exception e) {
			System.err.println("Errore irreversibile, il seguente: ");
			e.printStackTrace();
			System.err.println("Chiudo!");
			System.exit(4);
		}
	} // main
} // GetFileClient
