// PutFileServer Concorrente

import java.io.*;
import java.net.*;

// Thread lanciato per ogni richiesta accettata
// versione per il trasferimento di file binari
class GetFileServerThread extends Thread {

	private Socket clientSocket = null;

	// costruttore
	public GetFileServerThread(Socket clientSocket) {
		this.clientSocket = clientSocket;
	}

	/**
	 * Main invocabile con 0 o 1 argomenti. Argomento possibile -> porta su cui il
	 * server ascolta.
	 * 
	 */
	public void run() {
		DataInputStream inSock;
		DataOutputStream outSock;

		System.out.println("Attivazione figlio: "+Thread.currentThread().getId());
		try {
			try {
				// creazione stream di input e out da socket
				inSock = new DataInputStream(clientSocket.getInputStream());
				outSock = new DataOutputStream(clientSocket.getOutputStream());
			} catch (IOException ioe) {
				System.out
						.println("Problemi nella creazione degli stream di input/output "
								+ "su socket: ");
				ioe.printStackTrace();
				// il server continua l'esecuzione riprendendo dall'inizio del ciclo
				return;
			} catch (Exception e) {
				System.out
						.println("Problemi nella creazione degli stream di input/output "
								+ "su socket: ");
				e.printStackTrace();
				// il server continua l'esecuzione riprendendo dall'inizio del ciclo
				return;
			}

			/* ricezione file */
			String nomeDirettorio = null;
			try {
				// ricezione del nome
				nomeDirettorio = inSock.readUTF();
				File dirCorr = new File(nomeDirettorio);
				if ( dirCorr.exists() && dirCorr.isDirectory() ) {
					File[] files = dirCorr.listFiles();
					File fileCorr;
					for (int i = 0; i < files.length; i++) {
						fileCorr = files[i];
						System.out.println("File con nome: " + fileCorr.getName());
						if (fileCorr.isFile()) {
							// Trasmissione: nome file
							outSock.writeUTF(fileCorr.getName());
							// lunghezza
							outSock.writeLong(fileCorr.length());
							// trasferimento dati
							FileUtility.trasferisci_N_byte_file_binario(new DataInputStream(
									new FileInputStream(fileCorr.getAbsolutePath())), outSock, fileCorr
									.length());
						}
					}
				} else {
					// invio come messaggio di errore un carattere speciale
					outSock.writeUTF("?");
				}
				// chiudo la comunicazione col client
				clientSocket.close();
			} catch (SocketTimeoutException ste) {
				System.out.println("Timeout scattato: ");
				ste.printStackTrace();
				clientSocket.close();
			} catch (Exception e) {
				System.out.println("Problemi, i seguenti : ");
				e.printStackTrace();
				System.out.println("Chiudo ed esco...");
				clientSocket.close();
			}
		} catch (Exception e) {
			e.printStackTrace();
			// chiusura di stream e socket
			System.out
					.println("Errore irreversibile, GetFileServerThread: termino...");
		}
		System.out.println("Terminazione figlio: "+Thread.currentThread().getId());
	} // run

} // GetFileServerCon

public class GetFileServerCon {

	public static void main(String[] args) throws IOException {

		// Porta sulla quale ascolta il server
		int port = -1;

		/* controllo argomenti */
		try {
			if (args.length == 1) {
				port = Integer.parseInt(args[0]);
			} else {
				System.out.println("Usage: java GetFileServerCon port");
				System.exit(1);
			}
		} // try
		catch (Exception e) {
			System.out.println("Problemi, i seguenti: ");
			e.printStackTrace();
			System.out
					.println("Usage: java GetFileServerCon port");
			System.exit(1);
		}

		ServerSocket serverSocket = null;
		Socket clientSocket = null;

		try {
			serverSocket = new ServerSocket(port);
			serverSocket.setReuseAddress(true);
			System.out.println("GetFileServerCon: avviato ");
			System.out.println("Server: creata la server socket: " + serverSocket);
		} catch (Exception e) {
			System.err
					.println("Server: problemi nella creazione della server socket: "
							+ e.getMessage());
			e.printStackTrace();
			System.exit(1);
		}

		try {

			while (true) {
				System.out.println("Server: in attesa di richieste...\n");

				try {
					// bloccante finch non avviene una connessione
					clientSocket = serverSocket.accept();
					clientSocket.setSoTimeout(60000);
					// anche se il server e' concorrente e' comunque meglio evitare che
					// un blocco indefinito
					System.out.println("Server: connessione accettata: " + clientSocket);
				} catch (Exception e) {
					System.err
							.println("Server: problemi nella accettazione della connessione: "
									+ e.getMessage());
					e.printStackTrace();
					continue;
					// il server continua a fornire il servizio ricominciando dall'inizio
					// del ciclo
				}

				// serizio delegato ad un nuovo thread
				try {
					new GetFileServerThread(clientSocket).start();
				} catch (Exception e) {
					System.err.println("Server: problemi nel server thread: "
							+ e.getMessage());
					e.printStackTrace();
					continue;
					// il server continua a fornire il servizio ricominciando dall'inizio
					// del ciclo
				}

			} // while
		}
		// qui catturo le eccezioni non catturate all'interno del while
		// in seguito alle quali il server termina l'esecuzione
		catch (Exception e) {
			e.printStackTrace();
			// chiusura di stream e socket
			System.out.println("GetFileServerCon: termino...");
			System.exit(2);
		}

	}
} // GetFileServerCon
